{-# OPTIONS_GHC -optc-D__HUGS__ #-}
{-# INCLUDE <sql.h> #-}
{-# INCLUDE <sqlext.h> #-}
{-# LINE 1 "TypeConv.hsc" #-}
-- -*- mode: haskell; -*-
{-# LINE 2 "TypeConv.hsc" #-}
{-# CFILES hdbc-odbc-helper.c #-}
-- Above line for hugs
{-
Copyright (C) 2006 John Goerzen <jgoerzen@complete.org>

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
-}
module Database.HDBC.ODBC.TypeConv(fromOTypeInfo, fromOTypeCol) where
import Database.HDBC.Types
import Database.HDBC
import Database.HDBC.DriverUtils
import Database.HDBC.ODBC.Types
import Database.HDBC.ODBC.Utils
import Foreign.C.Types
import Foreign.ForeignPtr
import Foreign.Ptr
import Control.Concurrent.MVar
import Foreign.C.String
import Foreign.Marshal
import Foreign.Storable
import Control.Monad
import Data.List
import Data.Word
import Data.Int
import Control.Exception
import System.IO
import Data.Maybe

l _ = return ()
--l m = hPutStrLn stderr ("\n" ++ m)


{-# LINE 46 "TypeConv.hsc" #-}

{-# LINE 47 "TypeConv.hsc" #-}

fromOTypeInfo :: String         -- ^ Column name
              -> Int16 -- ^ Data type
{-# LINE 50 "TypeConv.hsc" #-}
              -> Word32 -- ^ Column size
{-# LINE 51 "TypeConv.hsc" #-}
              -> Int16 -- ^ Is it nullable
{-# LINE 52 "TypeConv.hsc" #-}
              -> (String, SqlColDesc)
fromOTypeInfo colname datatype colsize nullable =
    (colname,
     SqlColDesc {colType = convdatatype datatype,
                 colOctetLength = Nothing,
                 colDecDigits = Nothing,
                 colSize = Just (fromIntegral colsize),
                 colNullable = case nullable of
                                 0 -> Just False
{-# LINE 61 "TypeConv.hsc" #-}
                                 1 -> Just True
{-# LINE 62 "TypeConv.hsc" #-}
                                 _ -> Nothing
                }
    )

fromOTypeCol (_:_:_:colname:datatype:_:colsize:buflen:decdig:precrad:nullable:_:_:_:subtype:octetlen:_) =
    fromOTypeInfo (fromSql colname)
                  (fromIntegral ((fromSql datatype)::Int))
                  (fromSql colsize)
                  (fromIntegral ((fromSql nullable)::Int))
fromOTypeCol x = error $ "fromOTypeCol: unexpected result set: " ++ show x

convdatatype :: Int16 -> SqlTypeId
{-# LINE 74 "TypeConv.hsc" #-}
convdatatype intype =
    case intype of
      1 -> SqlCharT
{-# LINE 77 "TypeConv.hsc" #-}
      12 -> SqlVarCharT
{-# LINE 78 "TypeConv.hsc" #-}
      -1 -> SqlLongVarCharT
{-# LINE 79 "TypeConv.hsc" #-}
      -8 -> SqlWCharT
{-# LINE 80 "TypeConv.hsc" #-}
      -9 -> SqlWVarCharT
{-# LINE 81 "TypeConv.hsc" #-}
      -10 -> SqlWLongVarCharT
{-# LINE 82 "TypeConv.hsc" #-}
      3 -> SqlDecimalT
{-# LINE 83 "TypeConv.hsc" #-}
      2 -> SqlNumericT
{-# LINE 84 "TypeConv.hsc" #-}
      5 -> SqlSmallIntT
{-# LINE 85 "TypeConv.hsc" #-}
      4 -> SqlIntegerT
{-# LINE 86 "TypeConv.hsc" #-}
      7 -> SqlRealT
{-# LINE 87 "TypeConv.hsc" #-}
      6 -> SqlFloatT
{-# LINE 88 "TypeConv.hsc" #-}
      8 -> SqlDoubleT
{-# LINE 89 "TypeConv.hsc" #-}
      -7 -> SqlBitT
{-# LINE 90 "TypeConv.hsc" #-}
      -6 -> SqlTinyIntT
{-# LINE 91 "TypeConv.hsc" #-}
      -5 -> SqlBigIntT
{-# LINE 92 "TypeConv.hsc" #-}
      -2 -> SqlBinaryT
{-# LINE 93 "TypeConv.hsc" #-}
      -3 -> SqlVarBinaryT
{-# LINE 94 "TypeConv.hsc" #-}
      -4 -> SqlLongVarBinaryT
{-# LINE 95 "TypeConv.hsc" #-}
      91 -> SqlDateT
{-# LINE 96 "TypeConv.hsc" #-}
      92 -> SqlTimeT
{-# LINE 97 "TypeConv.hsc" #-}
      93 -> SqlTimestampT
{-# LINE 98 "TypeConv.hsc" #-}
      -- ODBC libraries don't seem to define the UTC items
      -- #{const SQL_TYPE_UTCDATETIME} -> SqlUTCDateTimeT
      -- #{const SQL_TYPE_UTCTIME} -> SqlUTCTimeT
      102 -> SqlIntervalT SqlIntervalMonthT
{-# LINE 102 "TypeConv.hsc" #-}
      101 -> SqlIntervalT SqlIntervalYearT
{-# LINE 103 "TypeConv.hsc" #-}
      107 -> SqlIntervalT SqlIntervalYearToMonthT
{-# LINE 104 "TypeConv.hsc" #-}
      103 -> SqlIntervalT SqlIntervalDayT
{-# LINE 105 "TypeConv.hsc" #-}
      104 -> SqlIntervalT SqlIntervalHourT
{-# LINE 106 "TypeConv.hsc" #-}
      105 -> SqlIntervalT SqlIntervalMinuteT
{-# LINE 107 "TypeConv.hsc" #-}
      106 -> SqlIntervalT SqlIntervalSecondT
{-# LINE 108 "TypeConv.hsc" #-}
      108 -> SqlIntervalT SqlIntervalDayToHourT
{-# LINE 109 "TypeConv.hsc" #-}
      109 -> SqlIntervalT SqlIntervalDayToMinuteT
{-# LINE 110 "TypeConv.hsc" #-}
      110 -> SqlIntervalT SqlIntervalDayToSecondT
{-# LINE 111 "TypeConv.hsc" #-}
      111 -> SqlIntervalT SqlIntervalHourToMinuteT
{-# LINE 112 "TypeConv.hsc" #-}
      112 -> SqlIntervalT SqlIntervalHourToSecondT
{-# LINE 113 "TypeConv.hsc" #-}
      113 -> SqlIntervalT SqlIntervalMinuteToSecondT
{-# LINE 114 "TypeConv.hsc" #-}
      -11 -> SqlGUIDT
{-# LINE 115 "TypeConv.hsc" #-}
      x -> SqlUnknownT (show x)
